@@ -196,9 +196,9 @@ var gDataMigrator = {
         migLOG("DMs: " + DMs.length);
 
         var url = "chrome://calendar/content/calendar-migration-dialog.xul";
-//@line 207 "/usr/obj/seamonkey-2.33.1/comm-release/calendar/base/content/dialogs/calendar-migration-dialog.js"
+//@line 207 "/usr/obj/ports/seamonkey-2.39/seamonkey-2.39/calendar/base/content/dialogs/calendar-migration-dialog.js"
         openDialog(url, "migration", "modal,centerscreen,chrome,resizable=no,width=500,height=400", DMs);
-//@line 209 "/usr/obj/seamonkey-2.33.1/comm-release/calendar/base/content/dialogs/calendar-migration-dialog.js"
+//@line 209 "/usr/obj/ports/seamonkey-2.39/seamonkey-2.39/calendar/base/content/dialogs/calendar-migration-dialog.js"
     },
 
     /**
@@ -370,7 +370,7 @@ var gDataMigrator = {
                         endIndex = otherEnd;
                     }
                     var sub = str.substring(index, endIndex);
-                    str = str.replace(sub, "", "g");
+                    str = str.split(sub).join("");
                     index = str.indexOf(";TZID=");
                 }
                 var tempFile = gDataMigrator.dirService.get("TmpD", Components.interfaces.nsIFile);
@@ -454,72 +454,75 @@ var gDataMigrator = {
     },
 
     checkWindowsMail: function gdm_windowsMail() {
-        let maildir = this.dirService.get("LocalAppData",
-                                          Components.interfaces.nsILocalFile);
-        if (!maildir || !maildir.exists()) {
+        function doMigrate(aCalendarNodes, aMailDir, aCallback) {
+            let calManager = cal.getCalendarManager();
+
+            for (let node of aCalendarNodes) {
+                let name = node.getElementsByTagName("Name")[0].textContent;
+                let color = node.getElementsByTagName("Color")[0].textContent;
+                let enabled = node.getElementsByTagName("Enabled")[0].textContent == "True";
+
+                // The name is quoted, and the color also contains an alpha
+                // value. Lets just ignore the alpha value and take the
+                // color part.
+                name = name.replace(/(^'|'$)/g, "");
+                color = color.replace(/0x[0-9a-fA-F]{2}([0-9a-fA-F]{4})/, "#$1");
+
+                let calfile = aMailDir.clone();
+                calfile.append(name + ".ics");
+
+                if (calfile.exists()) {
+                    let storage = gDataMigrator.importICSToStorage(calfile)
+                    storage.name = name;
+
+                    if (color) {
+                        storage.setProperty("color", color);
+                    }
+                    calManager.registerCalendar(storage);
+
+                    if (enabled) {
+                        getCompositeCalendar().addCalendar(storage);
+                    }
+                }
+            }
+            aCallback();
+        }
+
+        if (!this.dirService.has("LocalAppData")) {
             // We are probably not on windows
             return [];
         }
+
+        let maildir = this.dirService.get("LocalAppData",
+                                          Components.interfaces.nsILocalFile);
+
         maildir.append("Microsoft");
         maildir.append("Windows Calendar");
         maildir.append("Calendars");
 
         let settingsxml = maildir.clone();
         settingsxml.append("Settings.xml");
-        if (!settingsxml || !settingsxml.exists()) {
-            // No Settings.xml, maybe Windows Calendar was never started?
-            return [];
-        }
-        let settingsXmlUri = Services.io.newFileURI(settingsxml);
 
-        let req = new XMLHttpRequest();
-        req.open("GET", settingsXmlUri.spec, false);
-        req.send(null);
-        if (req.status == 0) {
-            // The file was found, it seems we are on windows vista.
-            let doc = req.responseXML;
-            let root = doc.documentElement;
+        let migrators = [];
+        if (settingsxml.exists()) {
+            let settingsXmlUri = Services.io.newFileURI(settingsxml);
 
-            // Get all calendar property tags and return the migrator.
-            let calendars = doc.getElementsByTagName("VCalendar");
-            function doMigrate(aCallback) {
-                for each (let node in Array.slice(calendars)) {
-                    let name = node.getElementsByTagName("Name")[0].textContent;
-                    let color = node.getElementsByTagName("Color")[0].textContent;
-                    let enabled = node.getElementsByTagName("Enabled")[0].textContent == "True";
+            let req = new XMLHttpRequest();
+            req.open("GET", settingsXmlUri.spec, false);
+            req.send(null);
+            if (req.status == 0) {
+                // The file was found, it seems we are on windows vista.
+                let doc = req.responseXML;
+                let root = doc.documentElement;
 
-                    // The name is quoted, and the color also contains an alpha
-                    // value. Lets just ignore the alpha value and take the
-                    // color part.
-                    name = name.replace(/(^'|'$)/g, "");
-                    color = color.replace(/0x[0-9a-fA-F]{2}([0-9a-fA-F]{4})/, "#$1");
-
-                    let calfile = maildir.clone();
-                    calfile.append(name + ".ics");
-
-                    if (calfile.exists()) {
-                        let storage = gDataMigrator.importICSToStorage(calfile)
-
-                        storage.name = name;
-
-                        if (color) {
-                            storage.setProperty("color", color);
-                        }
-                        let calManager = cal.getCalendarManager();
-                        calManager.registerCalendar(storage);
-
-                        if (enabled) {
-                            getCompositeCalendar().addCalendar(storage);
-                        }
-                    }
+                // Get all calendar property tags and return the migrator.
+                let calendars = doc.getElementsByTagName("VCalendar");
+                if (calendars.length > 0) {
+                    migrators = [new dataMigrator("Windows Calendar", doMigrate.bind(null, calendars, maildir))];
                 }
-                aCallback();
-            }
-            if (calendars.length > 0) {
-                return [new dataMigrator("Windows Calendar", doMigrate)];
             }
         }
-        return [];
+        return migrators;
     },
 
     /**
