@@ -4,6 +4,10 @@
 
 Components.utils.import("resource://gre/modules/Services.jsm");
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
+Components.utils.import("resource://gre/modules/Promise.jsm");
+Components.utils.import("resource://gre/modules/Task.jsm");
+Components.utils.import("resource://gre/modules/AddonManager.jsm");
+Components.utils.import("resource://calendar/modules/calAsyncUtils.jsm");
 
 var gLastShownCalendarView = null;
 
@@ -149,6 +153,9 @@ function ltnOnLoad(event) {
     // any load that occurs
     window.removeEventListener("load", ltnOnLoad, false);
 
+    // Check if the binary component was loaded
+    checkCalendarBinaryComponent();
+
     document.getElementById("calendarDisplayDeck").
       addEventListener("select", LtnObserveDisplayDeckChange, true);
 
@@ -183,8 +190,136 @@ function ltnOnLoad(event) {
         MailToolboxCustomizeDone(aEvent, "CustomizeTaskToolbar");
     };
 
-    Services.obs.notifyObservers(window, "lightning-startup-done", false);
+    ltnIntegrationCheck();
 
+    Services.obs.notifyObservers(window, "lightning-startup-done", false);
+}
+
+/**
+ * Displays the Lightning integration notification bar
+ */
+function ltnIntegrationNotification() {
+    const kOptOut = "mail.calendar-integration.opt-out"; // default: false
+    const kNotify = "calendar.integration.notify"; // default: true
+    const kSupportUri = "https://support.mozilla.org/kb/thunderbird-calendar-integration";
+    const kLightningGuuid = "{e2fda1a4-762b-4020-b5ad-a41df1933103}";
+
+    // we fall back to messagepanebox for Seamonkey
+    let notifyBox = document.getElementById("mail-notification-box") ||
+                    document.getElementById("messagepanebox");
+
+    let appBrand = cal.calGetString("brand", "brandShortName", null, "branding");
+    let ltnBrand = ltnGetString("lightning", "brandShortName");
+    let label = ltnGetString("lightning", "integrationLabel", [appBrand, ltnBrand]);
+
+    // call backs for doing/undoing Lightning removal
+    let cbRemoveLightning = function (aAddon) {
+        aAddon.userDisabled = true;
+    };
+    let cbUndoRemoveLightning = function (aAddon) {
+        aAddon.userDisabled = false;
+    };
+
+    // call backs for the undo opt-out bar
+    let cbRestartNow = function(aNotificationBar, aButton) {
+        Services.startup.quit(Components.interfaces.nsIAppStartup.eRestart |
+                              Components.interfaces.nsIAppStartup.eForceQuit);
+    };
+    let cbUndoOptOut = function(aNotificationBar, aButton) {
+        Preferences.set(kNotify, true);
+        Preferences.set(kOptOut, false);
+        AddonManager.getAddonByID(kLightningGuuid, cbUndoRemoveLightning);
+        // display notification bar again
+        ltnIntegrationNotification();
+    };
+
+    // call backs for the opt-out bar
+    let cbLearnMore = function(aNotificationBar, aButton) {
+        // In SeaMonkey the second parameter should be either null or an
+        // event object with a non null target.ownerDocument.
+        openUILink(kSupportUri, { button: 0,
+                                  target: { ownerDocument: document } });
+        return true;
+    };
+    let cbKeepIt = function(aNotificationBar, aButton) {
+        Preferences.set(kNotify, false);
+    };
+    let cbOptOut = function(aNotificationBar, aButton) {
+        Preferences.set(kNotify, false);
+        Preferences.set(kOptOut, true);
+        AddonManager.getAddonByID(kLightningGuuid, cbRemoveLightning);
+        // let the user know that removal will be applied after restart
+        let restartLabel = ltnGetString("lightning", "integrationRestartLabel",[ltnBrand, appBrand]);
+        let button = [{
+             label:     ltnGetString("lightning", "integrationUndoButton"),
+             accessKey: ltnGetString("lightning", "integrationUndoAccessKey"),
+             popup:     null,
+             callback:  cbUndoOptOut
+         }, {
+             label:     ltnGetString("lightning", "integrationRestartButton"),
+             accessKey: ltnGetString("lightning", "integrationRestartAccessKey"),
+             popup:     null,
+             callback:  cbRestartNow
+         }];
+         notifyBox.appendNotification(restartLabel,
+                                      "restart-required",
+                                      null,
+                                      notifyBox.PRIORITY_INFO_MEDIUM,
+                                      button);
+    };
+
+    let buttons = [{
+         label:     ltnGetString("lightning", "integrationLearnMoreButton"),
+         accessKey: ltnGetString("lightning", "integrationLearnMoreAccessKey"),
+         popup:     null,
+         callback:  cbLearnMore
+    }, {
+        label:     ltnGetString("lightning", "integrationOptOutButton"),
+        accessKey: ltnGetString("lightning", "integrationOptOutAccessKey"),
+        popup:     null,
+        callback:  cbOptOut
+    }, {
+        label:     ltnGetString("lightning", "integrationKeepItButton"),
+        accessKey: ltnGetString("lightning", "integrationKeepItAccessKey"),
+        popup:     null,
+        callback:  cbKeepIt
+    }];
+
+    // we use PRIORITY_INFO_MEDIUM to overrule notifications from specialTabs.js if any
+    let notification = notifyBox.appendNotification(label,
+                                                    "calendar-integration",
+                                                    null,
+                                                    notifyBox.PRIORITY_INFO_MEDIUM,
+                                                    buttons);
+    notification.persistence = 3;
+}
+
+/**
+ * Checks whether to display the opt-out notification for Lightning integration
+ */
+function ltnIntegrationCheck() {
+    const kOptOut = "mail.calendar-integration.opt-out"; // default: false
+    const kNotify = "calendar.integration.notify"; // default: true
+    // don't do anything if the opt-out pref doesn't exist or is enabled by the user or the user has
+    // already decided to keep Lightning
+    if (!Preferences.get(kOptOut, true) && Preferences.get(kNotify, false)) {
+        // action is only needed, if hasn't used Lightning before, so lets check whether this looks
+        // like a default calendar setup
+        let cnt = new Object();
+        let calMgr = cal.getCalendarManager();
+        let cals = calMgr.getCalendars(cnt);
+        let homeCalName = cal.calGetString("calendar", "homeCalendarName", null, "calendar");
+        if (cnt.value == 1 &&
+            calMgr.getCalendarPref_(cals[0], "type") == "storage" &&
+            calMgr.getCalendarPref_(cals[0], "name") == homeCalName) {
+            // this looks like a default setup, so let's see whether the calendar contains any items
+            let pCal = cal.async.promisifyCalendar(cals[0]);
+            // we look at all items at any time, but we can stop if the first item was found
+            // if we've found no items, we call ltnIntegrationNotification to display the bar
+            pCal.getItems(Components.interfaces.calICalendar.ITEM_FILTER_ALL_ITEMS, 1, null, null)
+                .then(function(aItems) {if (!aItems.length) {ltnIntegrationNotification()}});
+        }
+    }
 }
 
 /* Called at midnight to tell us to redraw date-specific widgets.  Do NOT call
@@ -466,10 +601,11 @@ function onToolbarsPopupShowingWithMode(aEvent, aInsertPoint) {
         return;
     }
 
-    let toolbox = [gCurrentMode + "-toolbox"];
+    let toolbox = [];
     if (gCurrentMode != "mail") {
         toolbox.push("navigation-toolbox");
     }
+    toolbox.push(gCurrentMode + "-toolbox");
     onViewToolbarsPopupShowing(aEvent, toolbox, aInsertPoint);
 }
 
@@ -485,3 +621,53 @@ function InitViewCalendarPaneMenu() {
                             !calSidebar.getAttribute("collapsed"));
     }
 }
+
+/**
+ * Checks if Lightning's binary component was successfully loaded.
+ */
+function checkCalendarBinaryComponent() {
+    // Don't even get started if we are running ical.js or the binary component
+    // was successfully loaded.
+    if ("@mozilla.org/calendar/datetime;1" in Components.classes ||
+        Preferences.get("calendar.icaljs", false)) {
+        return;
+    }
+
+    const THUNDERBIRD_GUID = "{3550f703-e582-4d05-9a08-453d09bdfdc6}";
+    const SEAMONKEY_GUID = "{92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}";
+    const LIGHTNING_GUID = "{e2fda1a4-762b-4020-b5ad-a41df1933103}";
+
+    AddonManager.getAddonByID(LIGHTNING_GUID, (ext) => {
+        if (!ext) {
+            return;
+        }
+
+        let version;
+        let appversion = Services.appinfo.version;
+        let versionparts = appversion.split(".");
+        let extbrand = ltnGetString("lightning", "brandShortName");
+
+        switch (Services.appinfo.ID) {
+            case THUNDERBIRD_GUID: // e.g. 31.4.0 -> 3.3
+                version = ((parseInt(versionparts[0], 10) + 2) / 10).toFixed(1);
+                break;
+            case SEAMONKEY_GUID: // e.g. 2.28.4 -> 3.3
+                version = ((parseInt(versionparts[1], 10) + 5) / 10).toFixed(1);
+                break;
+        }
+
+        let text;
+        if (version && version != ext.version) {
+            let args = [extbrand, ext.version, version];
+            text = ltnGetString("lightning", "binaryComponentKnown", args);
+        } else {
+            let brand = cal.calGetString("brand", "brandShortName", null, "branding");
+            let args = [extbrand, brand, appversion, ext.version];
+            text = ltnGetString("lightning", "binaryComponentUnknown", args);
+        }
+
+        let title = ltnGetString("lightning", "binaryComponentTitle", [extbrand]);
+        openAddonsMgr("addons://detail/" + encodeURIComponent(LIGHTNING_GUID));
+        Services.prompt.alert(window, title, text);
+    });
+}
