@@ -6,6 +6,7 @@ Components.utils.import("resource:///modules/mailServices.js");
 Components.utils.import("resource://calendar/modules/calUtils.jsm");
 Components.utils.import("resource://gre/modules/Services.jsm");
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
+Components.utils.import("resource://gre/modules/Preferences.jsm");
 
 function convertFromUnicode(aCharset, aSrc) {
     let unicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
@@ -93,7 +94,7 @@ calItipEmailTransport.prototype = {
                         att = item.getAttendeeById("mailto:" + aItipItem.identity);
                     }
                     if (!att) { // should not happen anymore
-                        return;
+                        return false;
                     }
 
                     // work around BUG 351589, the below just removes RSVP:
@@ -128,11 +129,12 @@ calItipEmailTransport.prototype = {
                 }
             }
 
-            this._sendXpcomMail(aRecipients, aSubject, aBody, aItipItem);
+            return this._sendXpcomMail(aRecipients, aSubject, aBody, aItipItem);
         } else {
             // Sunbird case: Call user's default mailer on system.
             throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
         }
+        return false;
     },
 
     _initEmailTransport: function cietIES() {
@@ -190,7 +192,7 @@ calItipEmailTransport.prototype = {
                         "This will disable OL (up to 2003) to consume the mail as an iTIP invitation showing\n" +
                         "the usual calendar buttons.");
                 // To somehow have a last resort before sending spam, the user can choose to send the mail.
-                let prefCompatMode = cal.getPrefSafe("calendar.itip.compatSendMode", 0);
+                let prefCompatMode = Preferences.get("calendar.itip.compatSendMode", 0);
                 let inoutCheck = { value: (prefCompatMode == 1) };
                 let parent = Services.wm.getMostRecentWindow(null);
                 if (parent.closed) {
@@ -209,31 +211,56 @@ calItipEmailTransport.prototype = {
                 } // else go on with auto sending for now
                 compatMode = (inoutCheck.value ? 1 : 0);
                 if (compatMode != prefCompatMode) {
-                    cal.setPref("calendar.itip.compatSendMode", compatMode);
+                    Preferences.set("calendar.itip.compatSendMode", compatMode);
                 }
             }
             case (Components.interfaces.calIItipItem.AUTO): {
-                cal.LOG("sendXpcomMail: Found AUTO autoResponse type.");
-                let toList = "";
-                for each (let recipient in aToList) {
-                    // Strip leading "mailto:" if it exists.
-                    let rId = recipient.id.replace(/^mailto:/i, "");
-                    // Prevent trailing commas.
-                    if (toList.length > 0) {
-                        toList += ", ";
-                    }
-                    // Add this recipient id to the list.
-                    toList += rId;
+                // don't show log message in case of falling through
+                if (aItem.autoResponse == Components.interfaces.calIItipItem.AUTO) {
+                    cal.LOG("sendXpcomMail: Found AUTO autoResponse type.");
                 }
-                let mailFile = this._createTempImipFile(compatMode, toList, aSubject, aBody, aItem, identity);
+                let cbEmail = function (aVal, aInd, aArr) {
+                    let email = cal.getAttendeeEmail(aVal, true);
+                    if (!email.length) {
+                        cal.LOG("Invalid recipient for email transport: " + aVal.toString());
+                    }
+                    return email;
+                }
+                let toMap = aToList.map(cbEmail).filter(function (aVal, aInd, aArr) {return (aVal.length)});
+                if (toMap.length < aToList.length) {
+                    // at least one invalid recipient, so we skip sending for this message
+                    return false;
+                }
+                let toList = toMap.join(', ');
+                let composeUtils = Components.classes["@mozilla.org/messengercompose/computils;1"]
+                                             .createInstance(Components.interfaces.nsIMsgCompUtils);
+                let messageId = composeUtils.msgGenerateMessageId(identity);
+                let mailFile = this._createTempImipFile(compatMode, toList, aSubject, aBody, aItem, identity, messageId);
                 if (mailFile) {
                     // compose fields for message: from/to etc need to be specified both here and in the file
                     let composeFields = Components.classes["@mozilla.org/messengercompose/composefields;1"]
                                                   .createInstance(Components.interfaces.nsIMsgCompFields);
                     composeFields.characterSet = "UTF-8";
                     composeFields.to = toList;
-                    composeFields.from = identity.email;
+                    let mailfrom = (!identity.fullName.length) ? identity.mail : identity.fullName + " <" + identity.mail + ">";
+                    composeFields.from = (cal.validateRecipientList(mailfrom) == mailfrom)
+                                         ? mailfrom : identity.email;
                     composeFields.replyTo = identity.replyTo;
+                    composeFields.organization = identity.organization;
+                    composeFields.messageId = messageId;
+                    let validRecipients;
+                    if (identity.doCc) {
+                        validRecipients = cal.validateRecipientList(identity.doCcList);
+                        if (validRecipients != "") {
+                            composeFields.cc = validRecipients;
+                        }
+                    }
+                    if (identity.doBcc) {
+                        validRecipients = cal.validateRecipientList(identity.doBccList);
+                        if (validRecipients != "") {
+                            composeFields.bcc = validRecipients;
+                        }
+                    }
 
                     // xxx todo: add send/progress UI, maybe recycle
                     //           "@mozilla.org/messengercompose/composesendlistener;1"
@@ -253,6 +280,7 @@ calItipEmailTransport.prototype = {
                                             null  /* nsIMsgSendListener aListener */,
                                             null  /* nsIMsgStatusFeedback aStatusFeedback */,
                                             ""    /* password */);
+                    return true;
                 }
                 break;
             }
@@ -267,23 +295,25 @@ calItipEmailTransport.prototype = {
                                 "Unknown autoResponse type: " +
                                 aItem.autoResponse);
         }
+        return false;
     },
 
-    _createTempImipFile: function cietCTIF(compatMode, aToList, aSubject, aBody, aItem, aIdentity) {
+    _createTempImipFile: function (compatMode, aToList, aSubject, aBody, aItem, aIdentity, aMessageId) {
+        function encodeUTF8(text) {
+            return convertFromUnicode("UTF-8", text).replace(/(\r\n)|\n/g, "\r\n");
+        }
+        function encodeMimeHeader(header) {
+            let fieldNameLen = (header.indexOf(": ") + 2);
+            return MailServices.mimeConverter
+                               .encodeMimePartIIStr_UTF8(header,
+                                                         false,
+                                                         "UTF-8",
+                                                         fieldNameLen,
+                                                         Components.interfaces
+                                                                   .nsIMimeConverter
+                                                                   .MIME_ENCODED_WORD_SIZE);
+        }
         try {
-            function encodeUTF8(text) {
-                return convertFromUnicode("UTF-8", text).replace(/(\r\n)|\n/g, "\r\n");
-            }
-            function encodeMimeHeader(header) {
-                let fieldNameLen = (header.indexOf(": ") + 2);
-                return MailServices.mimeConverter
-                                   .encodeMimePartIIStr_UTF8(header,
-                                                             false,
-                                                             "UTF-8",
-                                                             fieldNameLen,
-                                                             Components.interfaces.nsIMimeConverter.MIME_ENCODED_WORD_SIZE);
-            }
-
             let itemList = aItem.getItemList({});
             let serializer = Components.classes["@mozilla.org/calendar/ics-serializer;1"]
                                        .createInstance(Components.interfaces.calIIcsSerializer);
@@ -294,16 +324,35 @@ calItipEmailTransport.prototype = {
             let calText = serializer.serializeToString();
             let utf8CalText = encodeUTF8(calText);
 
+            let fullFrom = !aIdentity.fullName.length ? null :
+                           cal.validateRecipientList(aIdentity.fullName + "<" + aIdentity.email + ">");
+
             // Home-grown mail composition; I'd love to use nsIMimeEmitter, but it's not clear to me whether
             // it can cope with nested attachments,
             // like multipart/alternative with enclosed text/calendar and text/plain.
             let mailText = ("MIME-version: 1.0\r\n" +
                             (aIdentity.replyTo
                              ? "Return-path: " + aIdentity.replyTo + "\r\n" : "") +
-                            "From: " + aIdentity.email + "\r\n" +
+                            "From: " + (fullFrom || aIdentity.email) + "\r\n" +
+                            (aIdentity.organization
+                             ? "Organization: " + aIdentity.organization + "\r\n" : "") +
+                            "Message-ID: " + aMessageId + "\r\n" +
                             "To: " + aToList + "\r\n" +
                             "Date: " + (new Date()).toUTCString() + "\r\n" +
                             "Subject: " + encodeMimeHeader(aSubject.replace(/(\n|\r\n)/, "|")) + "\r\n");
+            let validRecipients;
+            if (aIdentity.doCc) {
+                validRecipients = cal.validateRecipientList(aIdentity.doCcList);
+                if (validRecipients != "") {
+                    mailText += ("Cc: " + validRecipients + "\r\n");
+                }
+            }
+            if (aIdentity.doBcc) {
+                validRecipients = cal.validateRecipientList(aIdentity.doBccList);
+                if (validRecipients != "") {
+                    mailText += ("Bcc: " + validRecipients + "\r\n");
+                }
+            }
             switch (compatMode) {
                 case 1:
                     mailText += ("Content-class: urn:content-classes:calendarmessage\r\n" +
